@@ -2,6 +2,7 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
+// @dart = 2.6
 
 // Conversions for IDBKey.
 //
@@ -39,7 +40,6 @@ convertNativeToDart_SerializedScriptValue(object) {
   return convertNativeToDart_AcceptStructuredClone(object, mustCopy: true);
 }
 
-
 /**
  * Converts a Dart value into a JavaScript SerializedScriptValue.  Returns the
  * original input or a functional 'copy'.  Does not mutate the original.
@@ -57,10 +57,9 @@ convertNativeToDart_SerializedScriptValue(object) {
  * its output, the result may share structure with the input [value].
  */
 abstract class _StructuredClone {
-
   // TODO(sra): Replace slots with identity hash table.
   var values = [];
-  var copies = [];  // initially 'null', 'true' during initial DFS, then a copy.
+  var copies = []; // initially 'null', 'true' during initial DFS, then a copy.
 
   int findSlot(value) {
     int length = values.length;
@@ -71,12 +70,19 @@ abstract class _StructuredClone {
     copies.add(null);
     return length;
   }
+
   readSlot(int i) => copies[i];
-  writeSlot(int i, x) { copies[i] = x; }
-  cleanupSlots() {}  // Will be needed if we mark objects with a property.
+  writeSlot(int i, x) {
+    copies[i] = x;
+  }
+
+  cleanupSlots() {} // Will be needed if we mark objects with a property.
   bool cloneNotRequired(object);
+  JSObject newJsObject();
+  void forEachObjectKey(object, action(key, value));
+  void putIntoObject(object, key, value);
   newJsMap();
-  newJsList(length);
+  List newJsList(length);
   void putIntoMap(map, key, value);
 
   // Returns the input, or a clone of the input.
@@ -127,21 +133,34 @@ abstract class _StructuredClone {
       // non-native properties or methods from interceptors and such, e.g.
       // an immutability marker. So we  had to stop doing that.
       var slot = findSlot(e);
-      var copy = readSlot(slot);
+      var copy = JS('returns:List|Null;creates:;', '#', readSlot(slot));
       if (copy != null) return copy;
       copy = copyList(e, slot);
       return copy;
     }
 
+    if (e is JSObject) {
+      var slot = findSlot(e);
+      var copy = readSlot(slot);
+      if (copy != null) return copy;
+      copy = newJsObject();
+      writeSlot(slot, copy);
+      // TODO: Consider inlining this so we don't allocate a closure.
+      forEachObjectKey(e, (key, value) {
+        putIntoObject(copy, key, walk(value));
+      });
+      return copy;
+    }
+
     throw new UnimplementedError('structured clone of other type');
   }
 
-  copyList(List e, int slot) {
+  List copyList(List e, int slot) {
     int i = 0;
     int length = e.length;
     var copy = newJsList(length);
     writeSlot(slot, copy);
-    for ( ; i < length; i++) {
+    for (; i < length; i++) {
       copy[i] = walk(e[i]);
     }
     return copy;
@@ -173,10 +192,9 @@ abstract class _StructuredClone {
  * the value as seen from the JavaScript listeners.
  */
 abstract class _AcceptStructuredClone {
-
   // TODO(sra): Replace slots with identity hash table.
   var values = [];
-  var copies = [];  // initially 'null', 'true' during initial DFS, then a copy.
+  var copies = []; // initially 'null', 'true' during initial DFS, then a copy.
   bool mustCopy = false;
 
   int findSlot(value) {
@@ -193,14 +211,16 @@ abstract class _AcceptStructuredClone {
   /// wrappers may not be identical, but their underlying Js Object might be.
   bool identicalInJs(a, b);
   readSlot(int i) => copies[i];
-  writeSlot(int i, x) { copies[i] = x; }
+  writeSlot(int i, x) {
+    copies[i] = x;
+  }
 
   /// Iterate over the JS properties.
-  forEachJsField(object, action);
+  forEachJsField(object, action(key, value));
 
   /// Create a new Dart list of the given length. May create a native List or
   /// a JsArray, depending if we're in Dartium or dart2js.
-  newDartList(length);
+  List newDartList(length);
 
   walk(e) {
     if (e == null) return e;
@@ -218,7 +238,7 @@ abstract class _AcceptStructuredClone {
     }
 
     if (isJavaScriptPromise(e)) {
-      return convertNativePromiseToDartFuture(e);
+      return promiseToFuture(e);
     }
 
     if (isJavaScriptSimpleObject(e)) {
@@ -235,24 +255,25 @@ abstract class _AcceptStructuredClone {
     }
 
     if (isJavaScriptArray(e)) {
-      var slot = findSlot(e);
-      var copy = readSlot(slot);
+      var l = JS<List>('returns:List;creates:;', '#', e);
+      var slot = findSlot(l);
+      var copy = JS<List>('returns:List|Null;creates:;', '#', readSlot(slot));
       if (copy != null) return copy;
 
-      int length = e.length;
+      int length = l.length;
       // Since a JavaScript Array is an instance of Dart List, we can modify it
       // in-place unless we must copy.
-      copy = mustCopy ? newDartList(length) : e;
+      copy = mustCopy ? newDartList(length) : l;
       writeSlot(slot, copy);
 
       for (int i = 0; i < length; i++) {
-        copy[i] = walk(e[i]);
+        copy[i] = walk(l[i]);
       }
       return copy;
     }
 
     // Assume anything else is already a valid Dart object, either by having
-    // already been processed, or e.g. a clonable native class.
+    // already been processed, or e.g. a cloneable native class.
     return e;
   }
 
@@ -275,9 +296,14 @@ class ContextAttributes {
   bool stencil;
   bool failIfMajorPerformanceCaveat;
 
-  ContextAttributes(this.alpha, this.antialias, this.depth,
-      this.failIfMajorPerformanceCaveat, this.premultipliedAlpha,
-      this.preserveDrawingBuffer, this.stencil);
+  ContextAttributes(
+      this.alpha,
+      this.antialias,
+      this.depth,
+      this.failIfMajorPerformanceCaveat,
+      this.premultipliedAlpha,
+      this.preserveDrawingBuffer,
+      this.stencil);
 }
 
 convertNativeToDart_ContextAttributes(nativeContextAttributes) {
@@ -307,7 +333,6 @@ class _TypedImageData implements ImageData {
 }
 
 ImageData convertNativeToDart_ImageData(nativeImageData) {
-
   // None of the native getters that return ImageData are declared as returning
   // [ImageData] since that is incorrect for FireFox, which returns a plain
   // Object.  So we need something that tells the compiler that the ImageData
@@ -317,7 +342,6 @@ ImageData convertNativeToDart_ImageData(nativeImageData) {
   JS('ImageData', '0');
 
   if (nativeImageData is ImageData) {
-
     // Fix for Issue 16069: on IE, the `data` field is a CanvasPixelArray which
     // has Array as the constructor property.  This interferes with finding the
     // correct interceptor.  Fix it by overwriting the constructor property.
@@ -346,8 +370,8 @@ ImageData convertNativeToDart_ImageData(nativeImageData) {
 // with native names.
 convertDartToNative_ImageData(ImageData imageData) {
   if (imageData is _TypedImageData) {
-    return JS('', '{data: #, height: #, width: #}',
-        imageData.data, imageData.height, imageData.width);
+    return JS('', '{data: #, height: #, width: #}', imageData.data,
+        imageData.height, imageData.width);
   }
   return imageData;
 }
